-- Copyright (c) 2006 Tobias Gubener
-- Subdesign CPC T-REX by TobiFlex
-- All rights reserved
--
-- Redistribution and use in source and synthezised forms, with or without
-- modification, are permitted provided that the following conditions are met:
--
-- Redistributions of source code must retain the above copyright notice,
-- this list of conditions and the following disclaimer.
--
-- Redistributions in synthesized form must reproduce the above copyright
-- notice, this list of conditions and the following disclaimer in the
-- documentation and/or other materials provided with the distribution.
--
-- Neither the name of the author nor the names of other contributors may
-- be used to endorse or promote products derived from this software without
-- specific prior written permission.
--
-- THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
-- AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
-- THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
-- PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE
-- LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
-- CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
-- SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
-- INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
-- CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
-- ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
-- POSSIBILITY OF SUCH DAMAGE.
--



library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_unsigned.all;

entity scope is
	generic(
		doubleline  : integer := 1;	-- 0 => single line, 1 => double line 	
		repeatvalue : integer := 1; -- 0 => show value once, 1 => repeat value
		xfirstpixel	: integer :=50	-- start disply after hsync
	);
	port
	(
	hsync_in	: in std_logic;
	vsync_in	: in std_logic;
	pixelclk	: in std_logic;
	bypass		: in std_logic:='1';
	sampleclk	: in std_logic;
	sampledata	: in std_logic_vector(7 downto 0);
	shot		: in std_logic:='1';
	run			: in std_logic:='1';
	direktdata	: in std_logic_vector(15 downto 0);
	pixel		: out std_logic;
	p_trigger	: out std_logic;
	p_oneshot	: out std_logic
	);
end;

architecture rtl of scope is

constant blank		:std_logic_vector(2 downto 0) := "000";
constant nibble		:std_logic_vector(2 downto 0) := "001";
constant byte		:std_logic_vector(2 downto 0) := "010";
constant word		:std_logic_vector(2 downto 0) := "011";
constant bin		:std_logic_vector(2 downto 0) := "100";

signal 	wraddress	: STD_LOGIC_VECTOR (9 DOWNTO 0);
signal	rdaddress	: STD_LOGIC_VECTOR (9 DOWNTO 0);
signal 	xpos		: STD_LOGIC_VECTOR (9 DOWNTO 0);
signal	ypos		: STD_LOGIC_VECTOR (8 DOWNTO 0);

signal dispin		: std_logic_vector (15 downto 0);
signal dispind		: std_logic_vector (15 downto 0);
signal dispmask		: std_logic_vector (15 downto 0);
signal hsyncd		: std_logic;
signal vsyncd		: std_logic;
signal rdena		: std_logic;
signal test			: std_logic;
signal disp			: std_logic_vector (2 downto 0);
signal trigger		: std_logic;
signal triggerd		: std_logic;
signal sample		: std_logic;
signal sampled		: std_logic;
signal sampleend	: std_logic;
signal oneshot		: std_logic;
signal datatmp		: STD_LOGIC_VECTOR (7 DOWNTO 0);
signal q			: STD_LOGIC_VECTOR (7 DOWNTO 0);
signal bcd			: STD_LOGIC_VECTOR (3 DOWNTO 0);
signal bcdpixel		: STD_LOGIC_VECTOR (3 DOWNTO 0);
signal bcdpix		: std_logic;
signal bcdpixcnt	: STD_LOGIC_VECTOR (5 DOWNTO 0);
signal bcdpixrst	: STD_LOGIC_VECTOR (5 DOWNTO 0);
signal bcddisp		: std_logic;
signal double		: std_logic;
signal compare		: std_logic;
signal direktdatad	: std_logic_vector(15 downto 0);

component scoperam 
	PORT
	(
		data		: IN STD_LOGIC_VECTOR (7 DOWNTO 0);
		rdaddress	: IN STD_LOGIC_VECTOR (9 DOWNTO 0);
		rdclock		: IN STD_LOGIC ;
		rdclocken	: IN STD_LOGIC  := '1';
		wraddress	: IN STD_LOGIC_VECTOR (9 DOWNTO 0);
		wrclock		: IN STD_LOGIC ;
		wrclocken	: IN STD_LOGIC  := '1';
		wren		: IN STD_LOGIC  := '1';
		q			: OUT STD_LOGIC_VECTOR (7 DOWNTO 0)
	);
END component;


begin

-------------------------------------------------------------------
-- Trigger
-------------------------------------------------------------------
	process (sampledata,shot) begin
			trigger <= '0';
--			IF 	(sampledata(2 downto 0)/="111")	AND (sampledata(7)='0' OR sampledata(6)='0') THEN
--			IF 	(sampledata(2 downto 0)/="111")	AND (sampledata(7)='0') THEN
			IF 	(sampledata(3)='0') THEN
				trigger <= '1';
			END IF;
	end process;

-------------------------------------------------------------------
-- Layout
-------------------------------------------------------------------
		process (ypos,q,direktdatad) begin
					CASE ypos(8 downto 3) IS
-- displaytyps are word, byte, nibble, bin					
						WHEN "000010" => dispin <= direktdatad;
										 disp <= word;
						WHEN "000011" => dispin(7 downto 0) <= q(7 downto 0);
										 disp <= byte;
						WHEN "000100" => dispin(3 downto 0) <= q(3 downto 0);
										 disp <= nibble;
						WHEN "000101" => dispin(0) <= q(5);
										 disp <= bin;
						WHEN "000110" => dispin(0) <= q(4);
										 disp <= bin;
						WHEN "000111" => dispin(0) <= q(3);
										 disp <= bin;
						WHEN "001000" => dispin(0) <= q(2);
										 disp <= bin;
						WHEN "001001" => dispin(0) <= q(1);
										 disp <= bin;
						WHEN "001010" => dispin(0) <= q(0);
										 disp <= bin;
						WHEN OTHERS  =>  disp <= blank;
					END CASE;	
	end process;
	
-------------------------------------------------------------------
-------------------------------------------------------------------

scoperam_inst : scoperam PORT MAP (
		data	 	=> datatmp,
		rdaddress	=> rdaddress,
		rdclock	 	=> pixelclk,
		rdclocken	=> rdena,
		wraddress	=> wraddress,
		wrclock	 	=> sampleclk,
--		wrclocken	=> rdena,
		wren	 	=> sample,
		q	 		=> q
	);
	
	pixel <= test when bypass='0' AND xpos >= xfirstpixel else '0';
	p_oneshot <= NOT oneshot;
	p_trigger <= trigger;
	
-------------------------------------------------------------------
-- Sample
-------------------------------------------------------------------
	process (triggerd,trigger,sample,sampleend,oneshot,run) begin
			IF ((triggerd='0' AND trigger='1' AND oneshot='0') OR sample='1') AND sampleend='0' THEN
				sample <= '1';
			ELSE	
				sample <= NOT run;
			END IF;
	end process;
	process (sampleclk) begin
			if (sampleclk'event and sampleclk='1') then
				sampled <= sample;
				datatmp <= sampledata;
				triggerd <= (trigger AND NOT oneshot) OR sample;
				IF (sampled='0' AND sample='1') OR run='0' THEN
					direktdatad <= direktdata;
				END IF;
				IF  sampleend='1' THEN
					oneshot <= '1';
				END IF;
				IF shot='0' THEN
					oneshot <= '0';
				END IF;
				IF  sample='1' THEN
					wraddress <= wraddress+1;
				ELSE
					wraddress <= (OTHERS =>'0');					
				END IF;
				IF wraddress = "1111111111" THEN
					sampleend <='1';
				ELSE	
					sampleend <='0';
				END IF;
			end if;
	end process;

-------------------------------------------------------------------
-- Display
-------------------------------------------------------------------
	process (pixelclk) begin
			if (pixelclk'event and pixelclk='1') then
				rdena <= NOT rdena;
				dispind <= dispin;
				IF rdena='1' OR true THEN  --remove "OR true" to halve the pixelclock
					hsyncd <= hsync_in;
					IF hsyncd /= hsync_in THEN
						xpos <= (OTHERS =>'0');
					ELSE
						xpos <= xpos+1;
						IF xpos < xfirstpixel THEN
							rdaddress <= (OTHERS =>'0');
						ELSE
							rdaddress <= rdaddress+1;
						END IF;
					END IF;
					IF hsyncd='0' AND hsync_in='1' THEN
						double <= NOT double;
						IF double='1' OR doubleline=0 THEN
							vsyncd <= vsync_in;
							IF vsyncd /= vsync_in THEN
								ypos <= (OTHERS =>'0');
							ELSE
								ypos <= ypos+1;
							END IF;
						END IF;
					END IF;
					
					CASE disp IS
						WHEN word|byte|nibble =>
									IF compare='0' AND (ypos(2 downto 0)/="111") THEN
										test <= '1';
									ELSE	
										test <= bcdpix;
									END IF;	
									IF compare='0'  OR (xfirstpixel=xpos) OR ((bcdpixcnt(5 downto 3)="111") AND repeatvalue=1)THEN
--									IF compare='0' THEN
										bcdpixcnt <= bcdpixrst;
									ELSE
										IF bcdpixcnt(2 downto 0)="101" THEN		--pixel zwischen den Zeichen
											IF (bcdpixcnt(5 downto 3)/="111") OR repeatvalue=1 THEN
												bcdpixcnt(5 downto 3) <= bcdpixcnt(5 downto 3)+1;
											END IF;
											bcdpixcnt(2 downto 0) <= "000";
										ELSE
											bcdpixcnt(2 downto 0) <= bcdpixcnt(2 downto 0)+1;
										END IF;	
									END IF;	
						WHEN bin =>
									CASE ypos(2 downto 0) IS
										WHEN "001" => test <= (dispin(0) XOR dispind(0)) OR dispin(0);
										WHEN "010"|"011"|"100"|"101" => test <= dispin(0) XOR dispind(0);
										WHEN "110" => test <= (dispin(0) XOR dispind(0)) OR NOT dispin(0) OR NOT (rdaddress(0) OR rdaddress(1) OR rdaddress(2) OR rdaddress(3));
										WHEN OTHERS => test <= '0';
									END CASE;
						WHEN OTHERS => test <= '0';
					END CASE;
				END IF;
			end if;
	end process;
	
	process (dispin,dispind,disp) begin
			CASE disp IS
				WHEN nibble => 	bcdpixrst <= "011000";
								IF dispind(3 downto 0) = dispin(3 downto 0) THEN
									compare <= '1';
								ELSE
									compare <= '0';
								END IF;		
				WHEN byte   =>  bcdpixrst <= "010000";
								IF dispind(7 downto 0) = dispin(7 downto 0) THEN
									compare <= '1';
								ELSE
									compare <= '0';
								END IF;		
				WHEN OTHERS =>  bcdpixrst <= "000000";
								IF dispind(15 downto 0) = dispin(15 downto 0) THEN
									compare <= '1';
								ELSE
									compare <= '0';
								END IF;		
			END CASE;
	end process;
	
	process (bcdpixcnt,dispin) begin
			CASE bcdpixcnt(4 downto 3) IS
				WHEN "11" => bcd  <= dispin(3 downto 0);
				WHEN "10" => bcd  <= dispin(7 downto 4);
				WHEN "01" => bcd  <= dispin(11 downto 8);
				WHEN OTHERS=> bcd  <= dispin(15 downto 12);
			END CASE;		
	end process;
	
	process (bcd,ypos,bcdpixel,bcdpixcnt) begin
			CASE (bcdpixcnt(5)&bcdpixcnt(2 downto 0)) IS
				WHEN"0001" => bcdpix <= bcdpixel(3);
				WHEN"0010" => bcdpix <= bcdpixel(2);
				WHEN"0011" => bcdpix <= bcdpixel(1);
				WHEN"0100" => bcdpix <= bcdpixel(0);
				WHEN OTHERS => bcdpix <= '0';
			END CASE;
			CASE (bcd&ypos(2 downto 0)) IS
				WHEN"0000000" => bcdpixel <= "0110";
				WHEN"0000001" => bcdpixel <= "1001";
				WHEN"0000010" => bcdpixel <= "1001";
				WHEN"0000011" => bcdpixel <= "1001";
				WHEN"0000100" => bcdpixel <= "1001";
				WHEN"0000101" => bcdpixel <= "1001";
				WHEN"0000110" => bcdpixel <= "0110";
				
				WHEN"0001000" => bcdpixel <= "0010";
				WHEN"0001001" => bcdpixel <= "0110";
				WHEN"0001010" => bcdpixel <= "1010";
				WHEN"0001011" => bcdpixel <= "0010";
				WHEN"0001100" => bcdpixel <= "0010";
				WHEN"0001101" => bcdpixel <= "0010";
				WHEN"0001110" => bcdpixel <= "0111";
				
				WHEN"0010000" => bcdpixel <= "0110";
				WHEN"0010001" => bcdpixel <= "1001";
				WHEN"0010010" => bcdpixel <= "0001";
				WHEN"0010011" => bcdpixel <= "0110";
				WHEN"0010100" => bcdpixel <= "1000";
				WHEN"0010101" => bcdpixel <= "1000";
				WHEN"0010110" => bcdpixel <= "1111";
				
				WHEN"0011000" => bcdpixel <= "0110";
				WHEN"0011001" => bcdpixel <= "1001";
				WHEN"0011010" => bcdpixel <= "0001";
				WHEN"0011011" => bcdpixel <= "0110";
				WHEN"0011100" => bcdpixel <= "0001";
				WHEN"0011101" => bcdpixel <= "1001";
				WHEN"0011110" => bcdpixel <= "0110";
				
				WHEN"0100000" => bcdpixel <= "0001";
				WHEN"0100001" => bcdpixel <= "0010";
				WHEN"0100010" => bcdpixel <= "0100";
				WHEN"0100011" => bcdpixel <= "1001";
				WHEN"0100100" => bcdpixel <= "1111";
				WHEN"0100101" => bcdpixel <= "0001";
				WHEN"0100110" => bcdpixel <= "0001";
				
				WHEN"0101000" => bcdpixel <= "1111";
				WHEN"0101001" => bcdpixel <= "1000";
				WHEN"0101010" => bcdpixel <= "1110";
				WHEN"0101011" => bcdpixel <= "0001";
				WHEN"0101100" => bcdpixel <= "0001";
				WHEN"0101101" => bcdpixel <= "1001";
				WHEN"0101110" => bcdpixel <= "0110";
				
				WHEN"0110000" => bcdpixel <= "0011";
				WHEN"0110001" => bcdpixel <= "0100";
				WHEN"0110010" => bcdpixel <= "1000";
				WHEN"0110011" => bcdpixel <= "1110";
				WHEN"0110100" => bcdpixel <= "1001";
				WHEN"0110101" => bcdpixel <= "1001";
				WHEN"0110110" => bcdpixel <= "0110";
				
				WHEN"0111000" => bcdpixel <= "1111";
				WHEN"0111001" => bcdpixel <= "0001";
				WHEN"0111010" => bcdpixel <= "0010";
				WHEN"0111011" => bcdpixel <= "0100";
				WHEN"0111100" => bcdpixel <= "0100";
				WHEN"0111101" => bcdpixel <= "0100";
				WHEN"0111110" => bcdpixel <= "0100";
				
				WHEN"1000000" => bcdpixel <= "0110";
				WHEN"1000001" => bcdpixel <= "1001";
				WHEN"1000010" => bcdpixel <= "1001";
				WHEN"1000011" => bcdpixel <= "0110";
				WHEN"1000100" => bcdpixel <= "1001";
				WHEN"1000101" => bcdpixel <= "1001";
				WHEN"1000110" => bcdpixel <= "0110";
				
				WHEN"1001000" => bcdpixel <= "0110";
				WHEN"1001001" => bcdpixel <= "1001";
				WHEN"1001010" => bcdpixel <= "1001";
				WHEN"1001011" => bcdpixel <= "0111";
				WHEN"1001100" => bcdpixel <= "0001";
				WHEN"1001101" => bcdpixel <= "0010";
				WHEN"1001110" => bcdpixel <= "1100";
				
				WHEN"1010000" => bcdpixel <= "0110";
				WHEN"1010001" => bcdpixel <= "1001";
				WHEN"1010010" => bcdpixel <= "1001";
				WHEN"1010011" => bcdpixel <= "1111";
				WHEN"1010100" => bcdpixel <= "1001";
				WHEN"1010101" => bcdpixel <= "1001";
				WHEN"1010110" => bcdpixel <= "1001";
				
				WHEN"1011000" => bcdpixel <= "1110";
				WHEN"1011001" => bcdpixel <= "1001";
				WHEN"1011010" => bcdpixel <= "1001";
				WHEN"1011011" => bcdpixel <= "1110";
				WHEN"1011100" => bcdpixel <= "1001";
				WHEN"1011101" => bcdpixel <= "1001";
				WHEN"1011110" => bcdpixel <= "1110";
				
				WHEN"1100000" => bcdpixel <= "0110";
				WHEN"1100001" => bcdpixel <= "1001";
				WHEN"1100010" => bcdpixel <= "1000";
				WHEN"1100011" => bcdpixel <= "1000";
				WHEN"1100100" => bcdpixel <= "1000";
				WHEN"1100101" => bcdpixel <= "1001";
				WHEN"1100110" => bcdpixel <= "0110";
				
				WHEN"1101000" => bcdpixel <= "1100";
				WHEN"1101001" => bcdpixel <= "1010";
				WHEN"1101010" => bcdpixel <= "1001";
				WHEN"1101011" => bcdpixel <= "1001";
				WHEN"1101100" => bcdpixel <= "1001";
				WHEN"1101101" => bcdpixel <= "1010";
				WHEN"1101110" => bcdpixel <= "1100";
				
				WHEN"1110000" => bcdpixel <= "1111";
				WHEN"1110001" => bcdpixel <= "1000";
				WHEN"1110010" => bcdpixel <= "1000";
				WHEN"1110011" => bcdpixel <= "1110";
				WHEN"1110100" => bcdpixel <= "1000";
				WHEN"1110101" => bcdpixel <= "1000";
				WHEN"1110110" => bcdpixel <= "1111";
				
				WHEN"1111000" => bcdpixel <= "1111";
				WHEN"1111001" => bcdpixel <= "1000";
				WHEN"1111010" => bcdpixel <= "1000";
				WHEN"1111011" => bcdpixel <= "1110";
				WHEN"1111100" => bcdpixel <= "1000";
				WHEN"1111101" => bcdpixel <= "1000";
				WHEN"1111110" => bcdpixel <= "1000";
				
				WHEN OTHERS => bcdpixel <= "0000";
			END CASE;
	end process;
end;
